' SMTP - Simple Mail Transfer Protocol
' This was tricky, so you may need to tweek it to suit your needs
' K.Bunch - 12/02/07

'Example: RetVal = SendSMTP(Servername, Username, Password, FromMe@Provider.com, ToSomeone@Provider.com, ToSomeoneCc@Provider.com, ToSomeoneBcc@Provider.com, This is the subject, This is the message, 0)

'Notes: 
' - Returns 1 if successful or 0 if failed
' - You must use the revised version of QBase64 (QBase64Rev.inc)
' - Error messages are displayed as they occur
' - If there is a problem with the server, then it will take a long time to timeout
' - Always use port 25 for email service (do not change this)
' - Will set off your firewall (so you will want to allow or unblock)
' - Change smtpVersion if necessary (not sure how many versions are available)
' - Servers such as Yahoo require premium service to allow third party SMTP services
' - Tested OK with Verizon, but had problems with GoDaddy, GMail, and Yahoo (see item above)

$INCLUDE "QBase64Rev.inc"

FUNCTION ErrorSMTP(Message AS STRING) AS INTEGER
    Messagedlg(Message, mtError, mbOK, 0)
    ErrorSMTP = 0
END FUNCTION

FUNCTION SendSMTP(Server AS STRING, Username AS STRING, Password AS STRING, From AS STRING, Recipient AS STRING, Cc AS STRING, Bcc AS STRING, Subject AS STRING, Message AS STRING, TextStyle AS INTEGER) AS INTEGER
    DEFINT smtpSockNum
    DEFINT smtpCnt
    DEFINT smtpAmount
    DEFINT smtpAuth
        smtpAuth = 0
    DEFINT smtpError
        smtpError = 0
    DEFSTR smtpRetStr
    DEFSTR smtpRcpt
    DEFSTR smtpExt(0 TO 10)
    DEFSTR smtpVersion
        smtpVersion = "1.0"
    
    DIM smtpSocket AS QSOCKET
    
    IF Server = "" THEN Messagedlg("You must enter a "+CHR$(34)+"Server"+CHR$(34)+" name.", mtWarning, mbOK, 0): SendSMTP = 0: EXIT FUNCTION
    IF From = "" THEN Messagedlg("You must enter who mail is "+CHR$(34)+"From"+CHR$(34)+".", mtWarning, mbOK, 0): SendSMTP = 0: EXIT FUNCTION
    IF Recipient = "" THEN Messagedlg("You must enter a mail "+CHR$(34)+"Recipient"+CHR$(34)+".", mtWarning, mbOK, 0): SendSMTP = 0: EXIT FUNCTION
    
    ''''Close QSOCKET If Already Open
    IF smtpSockNum > 0 THEN smtpSocket.Close(smtpSockNum)    
    
    ''''Connect To The Specified Server
    smtpSockNum = smtpSocket.Connect(Server, 25)
    IF smtpSockNum < 1 THEN
        SendSMTP = ErrorSMTP("Unable to connect to " + Server)
        EXIT FUNCTION
    ELSE
        smtpRetStr = smtpSocket.ReadLine(smtpSockNum)
    END IF
    ''''Check If OK
    IF MID$(smtpRetStr, 1, 3) <> "220" THEN SendSMTP = ErrorSMTP(smtpRetStr): EXIT FUNCTION
    
    ''''Clear Any Existing Extended Commands
    FOR smtpAmount = 0 TO 10 '(There Are More Than 10, But 10 Are The Most Common)
        smtpExt(smtpAmount) = ""
    NEXT smtpAmount

    ''''Get Extended Commands
    smtpSocket.WriteLine(smtpSockNum, "ehlo")
    smtpCnt = 0
    DO
        smtpRetStr = smtpSocket.ReadLine(smtpSockNum)
        IF MID$(smtpRetStr, 1, 3) <> "250" THEN smtpError = 1: EXIT DO
        smtpExt(smtpCnt) = smtpRetStr
        smtpCnt = smtpCnt + 1
        ''''Exit The Loop If QSocket Looses The Connection
        IF smtpSocket.ConnectionReady(smtpSockNum) < 1 THEN EXIT DO
    LOOP UNTIL smtpSocket.IsServerReady(smtpSockNum) < 1

    ' Common Extended Commands
    ' 250-ENHANCEDSTATUSCODES
    ' 250-PIPELINING
    ' 250-8BITMIME
    ' 250-SIZE
    ' 250-DSN
    ' 250-ETRN
    ' 250-AUTH LOGIN PLAIN
    ' 250-STARTTLS
    ' 250-DELIVERBY
    ' 250 HELP 
    
    ''''This Gets You Out Of The Loop First If There Is An Error (Keeps From Eating Memory)
    IF smtpError = 1 THEN SendSMTP = ErrorSMTP(smtpRetStr): EXIT FUNCTION
    
    ''''Check For StartTLS (If StartTLS Exists, Then This Must Be Next Command)
    FOR smtpAmount = 0 TO 10
        IF TALLY(LCASE$(smtpExt(smtpAmount)), "starttls") > 0 THEN
            smtpSocket.WriteLine(smtpSockNum, "starttls")
            smtpRetStr = smtpSocket.ReadLine(smtpSockNum)
            ''''Check If OK
            IF MID$(smtpRetStr, 1, 3) <> "220" THEN SendSMTP = ErrorSMTP(smtpRetStr): EXIT FUNCTION
            smtpAmount = 10
        END IF
    NEXT smtpAmount
    
    ''''Check For Authorization (If Auth Exists, Then You Must Enter a Username and Password)
    FOR smtpAmount = 0 TO 10
        IF TALLY(LCASE$(smtpExt(smtpAmount)), "auth") > 0 THEN
            smtpAuth = 1
            smtpAmount = 10
        END IF
    NEXT smtpAmount
    
    ''''If Authorization Required, Then Username/Password
    IF smtpAuth = 1 THEN
        IF Username = "" OR Password = "" THEN SendSMTP = ErrorSMTP("You must enter a Username and/or Password."): EXIT FUNCTION
        
        ''''Inform Server Username/Password Is Next
        smtpSocket.WriteLine(smtpSockNum, "auth login")
        smtpRetStr = smtpSocket.ReadLine(smtpSockNum)
        IF MID$(smtpRetStr, 1, 3) <> "334" THEN SendSMTP = ErrorSMTP(smtpRetStr): EXIT FUNCTION
        
        ''''Enter Username In Base64 Format
        smtpSocket.WriteLine(smtpSockNum, Base64Encode(Username))
        smtpRetStr = smtpSocket.ReadLine(smtpSockNum)
        IF MID$(smtpRetStr, 1, 3) <> "334" THEN SendSMTP = ErrorSMTP(smtpRetStr): EXIT FUNCTION
        
        ''''Enter Password In Base64 Format
        smtpSocket.WriteLine(smtpSockNum, Base64Encode(Password))
        smtpRetStr = smtpSocket.ReadLine(smtpSockNum)
        IF MID$(smtpRetStr, 1, 3) <> "235" THEN SendSMTP = ErrorSMTP(smtpRetStr): EXIT FUNCTION
    END IF
    
    ''''Enter From
    smtpSocket.WriteLine(smtpSockNum, "mail from: <" + From + ">")
    smtpRetStr = smtpSocket.ReadLine(smtpSockNum)
    IF MID$(smtpRetStr, 1, 3) <> "250" THEN SendSMTP = ErrorSMTP(smtpRetStr): EXIT FUNCTION
    
    ''''Enter Recipients
    smtpRcpt = ""
    IF Cc <> "" THEN Recipient = Recipient + "," + Cc
    IF Bcc <> "" THEN Recipient = Recipient + "," + Bcc
    FOR smtpAmount = 1 TO LEN(Recipient)
        IF MID$(REPLACESUBSTR$(Recipient, " ", ""), smtpAmount, 1) = "," OR smtpAmount = LEN(Recipient) THEN
            IF smtpAmount = LEN(Recipient) THEN
                smtpRcpt = smtpRcpt + MID$(REPLACESUBSTR$(Recipient, " ", ""), smtpAmount, 1)
            END IF
            smtpSocket.WriteLine(smtpSockNum, "rcpt to: <" + smtpRcpt + ">")
            smtpRetStr = smtpSocket.ReadLine(smtpSockNum)
            IF MID$(smtpRetStr, 1, 3) <> "250" THEN SendSMTP = ErrorSMTP(smtpRetStr): EXIT FUNCTION
            smtpRcpt = ""
        ELSE            
            smtpRcpt = smtpRcpt + MID$(REPLACESUBSTR$(Recipient, " ", ""), smtpAmount, 1)
        END IF
    NEXT smtpAmount
        
    ''''Tell Server Preparing to Send Data
    smtpSocket.WriteLine(smtpSockNum, "data")
    smtpRetStr = smtpSocket.ReadLine(smtpSockNum)
    IF MID$(smtpRetStr, 1, 3) <> "354" THEN SendSMTP = ErrorSMTP(smtpRetStr): EXIT FUNCTION
    
    ''''Send the Message
    smtpSocket.WriteLine(smtpSockNum, "subject: " + Subject)
    smtpSocket.WriteLine(smtpSockNum, "from: " + From)
    smtpSocket.WriteLine(smtpSockNum, "to: " + Recipient)
    smtpSocket.WriteLine(smtpSockNum, "cc: " + Cc)
    smtpSocket.WriteLine(smtpSockNum, "bcc: " + Bcc)
    ''''Check If Plain or Html Text
    IF TextStyle <> 0 THEN
        smtpSocket.WriteLine(smtpSockNum, "MIME-Version: " + smtpVersion)
        smtpSocket.WriteLine(smtpSockNum, "Content-type: text/html")
    END IF
    ''''You Must Enter A Blank Line Here
    smtpSocket.WriteLine(smtpSockNum, )
    smtpSocket.WriteLine(smtpSockNum, Message)
    ''''You Must Enter a "." at the End
    smtpSocket.WriteLine(smtpSockNum, ".")    
    smtpRetStr = smtpSocket.ReadLine(smtpSockNum)
    IF MID$(smtpRetStr, 1, 3) <> "250" THEN SendSMTP = ErrorSMTP(smtpRetStr): EXIT FUNCTION

    ''''Quit SMTP
    smtpSocket.WriteLine(smtpSockNum, "quit")
    
    ''''Close QSOCKET
    smtpSocket.Close(smtpSockNum)
    
    SendSMTP = 1
END FUNCTION
